// ahfgen
//
// Create a dummy (fixed poinitng) XMM attitude history file
//
// Based on code by Uwe Lammers
//
// Copyright 1998-1999 European Space Agency, written BY Giuseppe Vacanti
//

#include "common.h"
#include "param.h"
#include "Dal.h"
#include "STime.h"
#include "Celestial.h"
#include "CalOalUtility.h"
#include <SasDalUtils.h>

#include "ahfgen.h"

double f(double min, double max, double t, double diff)
{
	return DEG_TO_RAD * (min + (max-min)/diff * t);
}


void ahfgen()
{
  string ahf_name = stringParameter("ahf");
  STime start(stringParameter("tstart"));
  STime stop(stringParameter("tstop"));


//   const double ra0 = 30., ra1 = 180.,
//     dec0 = -20., dec1 = 0.,
//     pos0 = -99., pos1 = 0.;

  double ra0, ra1, dec0, dec1, pos0, pos1;

  ra0=realParameter("ra");
  ra1=realParameter("ra");
  dec0=realParameter("dec");
  dec1=realParameter("dec");
  pos0=realParameter("posangle");
  pos1=realParameter("posangle");


  const double granularity = 10.; // s
  double diff = stop - start;
  const double blockLen = diff;	// s

  unsigned nTicks = UNT(blockLen/granularity) + 2; // required number of records
  
  DataSet *set = dataSetServer->open(ahf_name, DataSet::Create);

  * set->addAttribute("ORIGIN", "processing site") = stringParameter("origin");
  * set->addAttribute("CATEGORY") = stringParameter("category");
  * set->addAttribute("FILENAME") = stringParameter("ahf");


  Table *tab = set->addTable("SCATS1", nTicks);

  * tab->addAttribute("TELESCOP") = "XMM";
  * tab->addAttribute("INSTRUME") = "SC"; // stringParameter("instrument");
  * tab->addAttribute("DATATYPE") = "ATTHIS.EL";
  * tab->addAttribute("OBS_ID", "Observation identifier") = stringParameter("obsid");
  * tab->addAttribute("DATE-OBS") = stringParameter("tstart");
  * tab->addAttribute("DATE-END") = stringParameter("tstop");


  vector<unsigned long> size; 
  size.push_back(19);

  Column *valtime = tab->addColumn("VALTIME", Dal::DString,
				   "data valid from this time", "", size),
    *valdur = tab->addColumn("VALDUR", Dal::Real32,
			     "duration of validity");
  size[0] = 11;
  Column *viewra = tab->addColumn("VIEWRA", Dal::DString,
				  "RA of viewing direction", "", size),
    *viewdec = tab->addColumn("VIEWDECL", Dal::DString,
			      "Dec of viewing direction", "", size),
    *astpos = tab->addColumn("ASTPOS", Dal::Real32,
			     "astronomical position angle" "deg");
  
  ColumnData *valtimeData = valtime->data(),
    *viewraData = viewra->data(),
    *viewdecData = viewdec->data();

  real32 *valdurData = valdur->data()->real32Data(),
    *astposData = astpos->data()->real32Data();
  
  using namespace SasDalUtils;
  //  unsigned nRow = 0;
  //  for (unsigned blockNr = 0; blockNr<4; ++blockNr)
    for (unsigned n=0; n<nTicks; ++n) {
      double toff = n * granularity;
      STime t = start + toff;
      setStringColumnRow(valtimeData, n, t.toString());
      valdurData[n] = granularity;
      setStringColumnRow(viewraData, n,
					 RightAscension(f(ra0, ra1, toff, diff)).toString(':'));
      setStringColumnRow(viewdecData, n,
					 Declination(f(dec0, dec1, toff, diff)).toString(':'));
      astposData[n] = RAD_TO_DEG * f(pos0, pos1, toff, diff);
      //      ++nRow;
    }
  
  dataSetServer->close(set);
}


