// phkgen
//
// Create MOS and PN periodic hk files.
//
// Table structure arbitrary as it is not specified in the ODF ICD
//
// Based on code by Uwe Lammers
//
// Copyright 1998-1999 European Space Agency, written BY Giuseppe Vacanti
//
#include "common.h"
#include "param.h"
#include "Dal.h"
#include "STime.h"
#include "CalOalUtility.h"
#include <SasDalUtils.h>

#define NUM_OF(a) (sizeof(a)/sizeof(a[0]))

#include "phkgen.h"

void phkgen()
{

  STime start(stringParameter("tstart"));
  STime stop(stringParameter("tstop"));
  
  double hk_period = realParameter("granularity");
  double delta = stop - start;
  unsigned int nrows = int(delta/hk_period) + 2;

  DataSet * set = dataSetServer->open(stringParameter("phkf"), Dal::Create);

  * set->addAttribute("ORIGIN", "processing site") = stringParameter("origin");
  * set->addAttribute("CATEGORY") = stringParameter("category");
  * set->addAttribute("FILENAME") = stringParameter("phkf");

  string extname = stringParameter("extname");
  string iname = stringParameter("instrument");
  string ftcoarsename = "FTCOARSE";
  string ftfinename = "FTFINE";
  if(iname == "EMOS1"){
	extname = "M1PEH1";
	ftcoarsename = "E1573";
	ftfinename = "EU573";
  } else if(iname == "EMOS2"){
	extname = "M2PEH1";
	ftcoarsename = "K1573";
	ftfinename = "KU573";
  } else if(iname == "EPN"){
	extname = "PNPMH1";
	ftcoarsename = "F1030";
	ftfinename = "FU036";
  } else if(iname == "RGS1")
	extname = "R1PFH1";
  else if(iname == "RGS2")
	extname = "R2PFH1";
  else
	Sas::warning("UnexpectedInstrument", "Extension name defaulting to " + extname);
  
  Table * tab = set->addTable(extname, nrows);

  * tab->addAttribute("TELESCOP") = "XMM";
  * tab->addAttribute("INSTRUME") = stringParameter("instrument");
  * tab->addAttribute("DATATYPE") = "PERHK.EL";
  * tab->addAttribute("OBS_ID", "Observation identifier") = stringParameter("obsid");
  * tab->addAttribute("EXP_ID", "Exposure identifier") = stringParameter("expid");
  * tab->addAttribute("DATE-OBS") = stringParameter("tstart");
  * tab->addAttribute("DATE-END") = stringParameter("tstop");
  
  vector<unsigned long> size; size.push_back(19);
  ColumnData *tcol = tab->addColumn("TIME", Dal::DString,
				    "time tag", "ERT", size)->data();
  
  // These two leak memory
  int32 *coarseobt = tab->addColumn(ftcoarsename, Dal::Int32,
				    "COBT coarse", "s")->data()->int32Data();
  uint16 *fineobt = tab->addColumn(ftfinename, Dal::Uint16,
				  "COBT fine", "1/65536 s")->data()->uint16Data();

  using namespace SasDalUtils;
  for (unsigned n=0; n < nrows; ++n) {
    STime t = start + 10 * n;
    setStringColumnRow(tcol, n, t.toString());
    coarseobt[n] = 0;
    fineobt[n] = 21845;
  }
  
  dataSetServer->close(set);
}


