//
// tcfgen -- generates a dummy Time Correlation File
//
// Based on code by Uwe Lammers
//
// Copyright 1998-1999 European Space Agency, written by Giuseppe Vacanti
//

#include "common.h"
#include "param.h"
#include "CalOalUtility.h"
#include "Dal.h"
#include "STime.h"
#include "Celestial.h"
#include <SasDalUtils.h>

#include "tcfgen.h"

void tcfgen()
{

  const unsigned int nrows = 2;
  vector<unsigned long> size;
  
  STime start(stringParameter("tstart"));
  STime stop(stringParameter("tstop"));

  STime ref("1958-01-01T00:00:00"); // ESOC reference UTC


  DataSet * set = dataSetServer->open(stringParameter("tcf"), DataSet::Create);

  * set->addAttribute("ORIGIN", "processing site") = stringParameter("origin");
  * set->addAttribute("CATEGORY") = stringParameter("category");
  * set->addAttribute("FILENAME") = stringParameter("tcf");

  Table *tab = set->addTable("SCTCS1", nrows);

  * tab->addAttribute("TELESCOP") = "XMM";
  * tab->addAttribute("INSTRUME") = "SC";
  * tab->addAttribute("DATATYPE") = "TIMCOR.EL";
  * tab->addAttribute("OBS_ID", "Observation identifier") = stringParameter("obsid");
  * tab->addAttribute("DATE-OBS") = stringParameter("tstart");
  * tab->addAttribute("DATE-END") = stringParameter("tstop");


  ColumnData * frame = tab->addColumn("FRAME", Dal::Int32)->data(),
    * obtcoars = tab->addColumn("OBTCOARS", Dal::Int32)->data(),
    * obtfine = tab->addColumn("OBTFINE", Dal::Uint16)->data(),
    * utcday = tab->addColumn("UTCDAY", Dal::Int16)->data(),
    * utcmilli = tab->addColumn("UTCMILLI", Dal::Int32)->data(),
    * utcmicro = tab->addColumn("UTCMICRO", Dal::Int16)->data(),
    * utcflag = tab->addColumn("UTCFLAG", Dal::Int16)->data();
    
  size.push_back(39);

  ColumnData * corrpara = tab->addColumn("CORRPARA", Dal::DString, "", "", size)->data(),
    * corrparb = tab->addColumn("CORRPARB", Dal::DString, "", "", size)->data();

  size[0] = 23;
  ColumnData * ertime = tab->addColumn("ERTIME", Dal::DString, "", "", size)->data();
  
  size[0] = 5;
  ColumnData * gsid = tab->addColumn("GSID", Dal::DString, "", "", size)->data();

  int32 * frame_d = frame->int32Data(),
    * obtcoars_d = obtcoars->int32Data(),
    * utcmilli_d = utcmilli->int32Data();
  
  uint16 * obtfine_d = obtfine->uint16Data();
  int16 * utcday_d = utcday->int16Data(),
    * utcmicro_d = utcmicro->int16Data(),
    * utcflag_d = utcflag->int16Data();
  
  //   char * corrpara_d = corrpara->stringData(),
  //     * corrparb_d = corrparb->stringData(),	
  //     * errtime_d = errtime->stringData(),
  //     * gsid_d = gsid->stringData();
  

  // The FRAME column is not use by the OAL, so set it to 0
  frame_d[0] = 0;
  frame_d[1] = 0;

  // when all the counters on board are supposed to have been reset
  obtcoars_d[0] = 0;
  obtfine_d[0] = 0;
  
  // Now find the number of days from January 1, 1958 00:00:00 to the
  // modified start
  utcday_d[0] = static_cast<int>((start - ref)/STime::secondsPerDay);
  // The number of milliseconds from the midnight before start to start
  utcmilli_d[0] = static_cast<int>((start - ref) - (start.leapSeconds() - ref.leapSeconds()) - utcday_d[0]*STime::secondsPerDay)*1000;
  // The number of microseconds from the second before start to start
  // is taken to be 0
  utcmicro_d[0] = 0;
  utcflag_d[0] = 0;
  using namespace SasDalUtils;
  setStringColumnRow(corrpara, 0, "corra");
  setStringColumnRow(corrparb, 0, "corrb");
  setStringColumnRow(ertime, 0, start.toString());
  setStringColumnRow(gsid, 0, "GSIDF");

  obtcoars_d[1] = static_cast<int>(stop - start + 0.5);
  obtfine_d[1] = 0;
  utcday_d[1] = static_cast<int>((stop - ref)/STime::secondsPerDay);;
  utcmilli_d[1] = static_cast<int>((stop - ref) - (start.leapSeconds() - ref.leapSeconds()) - utcday_d[1]*STime::secondsPerDay)*1000;
  utcmicro_d[1] = 0;
  utcflag_d[1] = 0;
  setStringColumnRow(corrpara, 1, "corra");
  setStringColumnRow(corrparb, 1, "corrb");
  setStringColumnRow(ertime, 1, stop.toString());
  setStringColumnRow(gsid, 1, "GSIDF");
  

  delete frame;
  delete obtcoars;
  delete obtfine;
  delete utcday;
  delete utcmilli;
  delete utcflag;
  delete corrpara;
  delete corrparb;
  delete ertime;
  delete gsid;
  
  
  dataSetServer->close(set);
}


