module thfgen_mod
  use dal
  use param
  use errorHandling
  implicit none

contains
  
subroutine thfgen

  use readhist_mod

  type(DataSetT) :: trSet
  type(TableT)   :: trTable
  type(ColumnT)  :: trColumn(9)
  real exposure
  character(len=128) :: filename, obs_id, exp_id

  real, parameter :: frameTime = 20.
  real :: shiftx, shifty
  integer :: dims(1)
  integer i, j, nFrames
  real :: r
  integer(kind=int32), pointer :: frame(:)
  integer(kind=int32), pointer :: nggs(:)
  integer(kind=int32), pointer :: dx(:)
  integer(kind=int32), pointer :: dy(:)
  integer(kind=int32), pointer :: roll(:)
  integer(kind=int32), pointer :: quality(:)
  integer(kind=int32), pointer :: gsx(:,:)
  integer(kind=int32), pointer :: gsy(:,:)
  integer(kind=int32), pointer :: gscts(:,:)

  !! This added by GV -- 1998 November 24
  ! read parameters
  ! -- August 1999: more parameters read below
  filename = stringParameter("thf")
  exposure = realParameter("exposure")
  obs_id = stringParameter("obsid")
  exp_id = stringParameter("expid")
  !!

  nFrames = nint(exposure/frameTime)

  trSet = dataSet(filename, create)
  trTable = addTable(trSet, "OMTHX1", nFrames)

  i = len_trim(filename)

  call setAttribute(trSet,"FILENAME",filename(i-30:i))
  call setAttribute(trSet, "ORIGIN", "XMM-SOC")

  call setAttribute(trTable,"FILENAME",filename(i-30:i))
  call setAttribute(trTable,"OBS_ID",obs_id)
  call setAttribute(trTable,"EXP_ID",exp_id)

  call setAttribute(trTable, "TELESCOP", "XMM")
  call setAttribute(trTable, "INSTRUME", "OM")
  call setAttribute(trTable, "DATATYPE", "TRH.EL")
  call setAttribute(trTable, "CHAIN", 1)
  call setAttribute(trTable, "TMEXP_ID", exp_id)
  call setAttribute(trTable, "FRMTIME", frameTime)
  call setAttribute(trTable, "DATE-OBS", stringParameter("tstart"))
  call setAttribute(trTable, "DATE-END", stringParameter("tstop"))
   

  trColumn(1) = addColumn(trTable, "FRAME", integer32)
  frame => int32data(trColumn(1))

  trColumn(2) = addColumn(trTable, "NGGS", integer32)
  nggs => int32data(trColumn(2))

  trColumn(3) = addColumn(trTable, "DX", integer32)
  dx => int32data(trColumn(3))


  trColumn(4) = addColumn(trTable, "DY", integer32)
  dy => int32data(trColumn(4))

  trColumn(5) = addColumn(trTable, "ROLL", integer32)
  roll => int32data(trColumn(5))

  trColumn(6) = addColumn(trTable, "QUALITY", integer32)
  quality => int32data(trColumn(6))

  dims(1) = 10

  trColumn(7) = addColumn(trTable, "GSX", integer32, dimensions=dims)
  gsx => int32Vectordata(trColumn(7))

  trColumn(8) = addColumn(trTable, "GSY", integer32, dimensions=dims)
  gsy  => int32Vectordata(trColumn(8))

  trColumn(9) = addColumn(trTable, "GSCTS", integer32, dimensions=dims)
  gscts  => int32Vectordata(trColumn(9))

  call random_seed

  shiftx = 0.
  shifty = 0.

  do i = 0, nFrames-1

     call random_number(shiftx)
     shiftx = 4.*shiftx - 2.
     call random_number(shifty)
     shifty = 4.*shifty - 2.

     frame(i) = i + 1
     nggs(i) = 10
     if (i .eq. 0) then
        dx(i)  = nint(shiftx)
        dy(i)  = nint(shifty)
     else
        dx(i) = dx(i-1) + nint(shiftx)
        dy(i) = dy(i-1) + nint(shifty)
     endif
     roll(i) = 0
     quality(i) = 0
     do j = 0, 9
        call random_number(r)
        r = r * 100. + 100.
        gscts(j,i) = nint(r)
        gsx(j,i) = dx(i)
        gsy(j,i) = dy(i)
     enddo

  enddo

  dx = dx * 1000
  dy = dy * 1000

  call release(trTable)
  call release(trSet)
end subroutine thfgen

end module thfgen_mod
