//
// XMM Science Analysis System
//
// Giuseppe Vacanti (European Space Agency) 
// March 02, 2000
//
#include <string>
#include <vector>
#include <iostream>

#include <param.h>
#include <metatask.h>

#include <Dal.h>
#include <odffix.h>
#include <OalServer.h>
#include <Odf.h>

#include <Directory.h>

#include <cassert>

int main() {
  string odfdir = "odf";

  char * f[] = { "PEH", "WDX", "TCS", "ATS", "PMH", "THX", "NPH" };
  vector<string> ftypes(f, f + 7);

  openParameters("odffix");
  setParameter("odfdir", odfdir);
  task(odffix);

  Directory d(odfdir);
  d.filter("????_??????????_SCX?????SUM.ASC");
  cout << "Checking if the ODF summary file was created ... ";
  assert( d.size() == 1);
  cout << "ok." << endl;

  for(unsigned i = 0; i < ftypes.size(); i++){
    d.filter("????_??????????_????????" + ftypes[i] + ".FIT");
    cout << "Checking if " + ftypes[i] + " files were created ... ";
    assert( d.size() > 0 );
    cout << "ok." << endl;
  }

  // now ask the OAL
  const Odf *odf = oalServer.newOdf(odfdir);
  assert(odf->orbitNr() == 1);
  assert(odf->proposalId() == "000000");
  assert(odf->observationId() == "0000");
  assert(odf->isSlew() == false);
  assert(odf->sets() == 60);
  assert(odf->observationStartTime().toString() == "1999-02-19T13:57:05.000"); 
  assert(odf->observationEndTime().toString() == "1999-02-19T13:58:45.000"); 

  const Exposure *e = odf->exposure(Xmm::EMOS1, Xmm::IMAGING, 1);
  list<const OdfSet *> ls = e->sets();
  bool gotit = false;
  for(list<const OdfSet *>::const_iterator k = ls.begin(); k != ls.end(); k++){
    cout << (*k)->basename() << endl;
    if((*k)->basename() == "0001_0000000000_M1X00170IME.FIT")
      gotit = true;
  }
  assert( gotit ); // file with scheduled flag = X is in the exposure

  assert( e->scheduledStartTime().toString() == "1999-02-19T13:57:05.000" ); // but is not considered when calculating the exposure start/end
  assert( e->scheduledEndTime().toString() == "1999-02-19T13:58:45.000" );
    }
